import 'package:flutter/material.dart';

const _ArrowWidth = 5.0; //箭头宽度
const _ArrowHeight = 10.0; //箭头高度
const _MinHeight = 30.0; //内容最小高度
const _MinWidth = 30.0; //内容最小宽度

class Bubble extends StatelessWidget {
  final BubbleDirection direction;
  final Radius? borderRadius;
  final Widget child;
  final Decoration? decoration;
  final EdgeInsetsGeometry? padding;
  final EdgeInsetsGeometry? margin;
  final BoxConstraints? constraints;
  final double? width;
  final double? height;
  final Alignment? alignment;
  final bool? left;

  const Bubble(
      {Key? key,
      this.direction = BubbleDirection.left,
      required this.child,
      this.decoration,
      this.padding,
      this.margin,
      this.constraints,
      this.width,
      this.height,
      this.alignment,
      this.borderRadius,
      this.left})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ClipPath(
      clipper: _BubbleClipper(
          direction, borderRadius ?? Radius.circular(0), this.left),
      child: Container(
        decoration: this.decoration,
        child: Container(
          alignment: this.alignment,
          width: this.width,
          height: this.height,
          constraints: (this.constraints ?? BoxConstraints())
              .copyWith(minHeight: _MinHeight, minWidth: _MinWidth),
          decoration: this.decoration,
          padding: this.padding,
          margin: this.margin,
          child: this.child,
        ),
      ),
    );
  }
}

///方向
enum BubbleDirection { left, top, right, bottom }

class _BubbleClipper extends CustomClipper<Path> {
  final BubbleDirection direction;
  final Radius radius;
  final bool? left;

  _BubbleClipper(this.direction, this.radius, this.left);

  @override
  Path getClip(Size size) {
    final path = Path();
    final path2 = Path();
    final centerPoint = (size.height / 2).clamp(_MinHeight / 2, _MinHeight / 2);
    double centerPointWidth = size.width / 2;
    if (this.direction == BubbleDirection.left) {
      //绘制左边三角形
      path.moveTo(0, centerPoint);
      path.lineTo(_ArrowWidth, centerPoint - _ArrowHeight / 2);
      path.lineTo(_ArrowWidth, centerPoint + _ArrowHeight / 2);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(
              _ArrowWidth, 0, (size.width - _ArrowWidth), size.height),
          radius));
      //合并
      path.addPath(path2, Offset(0, 0));
    } else if (this.direction == BubbleDirection.right) {
      //绘制右边三角形
      path.moveTo(size.width, centerPoint);
      path.lineTo(size.width - _ArrowWidth, centerPoint - _ArrowHeight / 2);
      path.lineTo(size.width - _ArrowWidth, centerPoint + _ArrowHeight / 2);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(0, 0, (size.width - _ArrowWidth), size.height),
          radius));
      //合并
      path.addPath(path2, Offset(0, 0));
    } else if (this.direction == BubbleDirection.top) {
      //绘制上边三角形
      path.moveTo(centerPointWidth, 0);
      path.lineTo(centerPointWidth - _ArrowHeight / 2, _ArrowWidth);
      path.lineTo(centerPointWidth + _ArrowHeight / 2, _ArrowWidth);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(
              0, _ArrowWidth, size.width, size.height + _ArrowWidth * 2),
          radius));
      //合并
      path.addPath(path2, Offset(0, 0));
    } else {
      //绘制下边三角形
      if (left == true) {
        centerPointWidth = centerPointWidth / 2;
      } else if (left == false) {
        centerPointWidth = centerPointWidth + centerPointWidth / 2;
      }
      path.moveTo(centerPointWidth, size.height);
      path.lineTo(
          centerPointWidth - _ArrowHeight / 2, size.height - _ArrowWidth);
      path.lineTo(
          centerPointWidth + _ArrowHeight / 2, size.height - _ArrowWidth);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(0, 0, size.width, size.height - _ArrowWidth), radius));
      //合并
      path.addPath(path2, Offset(0, 0));
    }
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return false;
  }
}
